@@ -30,6 +30,7 @@ import logging
 import signal
 import sys
 import traceback
+from re import compile as re_compile
 from typing import Any, Callable, Coroutine, Dict, Generator, List, Literal, Optional, Sequence, TYPE_CHECKING, Tuple, TypeVar, Union, overload
 
 import aiohttp
@@ -63,16 +64,28 @@ from .stage_instance import StageInstance
 from .threads import Thread
 from .sticker import GuildSticker, StandardSticker, StickerPack, _sticker_factory
 from .application_commands import ApplicationCommand
+from .message import PartialMessage
 
 if TYPE_CHECKING:
-    from .abc import SnowflakeTime, PrivateChannel, GuildChannel, Snowflake as DiscordObject
+    from .abc import (
+        SnowflakeTime,
+        PrivateChannel,
+        GuildChannel,
+        Snowflake as DiscordObject,
+    )
     from .channel import DMChannel
     from .message import Message
     from .member import Member
     from .voice_client import VoiceProtocol
     from .reaction import Reaction
     from .raw_models import RawReactionActionEvent
-    from .interactions import Interaction
+    from .interactions import (
+        Interaction,
+        CommandInteraction,
+        AutocompleteInteraction,
+        ModalInteraction,
+        ComponentInteraction,
+    )
     from .types.snowflake import Snowflake
 
 __all__ = (
@@ -342,6 +355,40 @@ class Client:
         return asyncio.create_task(wrapped, name=f'Novus: {event_name}')
 
     def dispatch(self, event: str, *args: Any, **kwargs: Any) -> None:
+        """
+
+        Dispatches an event to all listeners with the ``on_`` prefix.
+
+        This could be used to send an event to a custom handler or utilize the same code for different
+        (Discord) dispatched events such as handling checks before welcoming a user to a Guild.
+
+        See the :func:`~discord.Client.event` decorator or the :ref:`documentation below <discord-api-events>` for how
+        to listen for an event.
+
+        Examples
+        --------
+
+        Reacting to every message with the word ``hello``: ::
+
+            @client.event
+            async def on_message(message):
+                if message.content == 'hello':
+                    client.dispatch('react_hello', message)
+
+            @client.event
+            async def on_react_hello(message):
+                await message.add_reaction('👋')
+
+
+        Parameters
+        ----------
+        event: :class:`str`
+            The event name without the ``on_`` prefix to dispatch out to.
+        args
+            Arguments to pass to the listener(s) that handle the dispatched event.
+        kwargs
+            Keyword Arguments to pass to the listener(s) that handle the dispatched event.
+        """
         _log.debug('Dispatching event %s', event)
         method = 'on_' + event
 
@@ -744,6 +791,50 @@ class Client:
         """
         return PartialMessageable(state=self._connection, id=id, type=type)
 
+    def get_partial_message(self, *, channel_type: Optional[ChannelType] = None, message_id: Optional[int] = None,
+                            channel_id: Optional[int] = None, message_url: Optional[str] = None
+                            ) -> Optional[PartialMessage]:
+        """
+
+        Returns a :class:`~discord.PartialMessage` from a ``message_id`` and ``channel_id`` or a message url.
+
+        ``channel_type`` must be one of :class:`~discord.ChannelType.text`, :class:`~discord.ChannelType.news`,
+        :class:`~discord.ChannelType.private`, :class:`~discord.ChannelType.news_thread`,
+        :class:`~discord.ChannelType.public_thread` or :class:`~discord.ChannelType.private_thread`.
+
+        .. note::
+
+            ``message_url`` will override ``channel_id`` and ``message_id`` if provided
+
+        Returns
+        -------
+        Optional[:class:`~discord.PartialMessage`]
+            The partial message object from the given parameters or ``None`` if the url cannot be matched and
+            ``channel_id`` & ``message_id`` are not specified.
+        """
+        if message_url is None:
+            if channel_id is None or message_id is None:
+                return None
+        else:
+            url_regex = re_compile(
+                r'https?://(?:(ptb|canary|www)\.)?discord(?:app)?\.com/channels/'
+                r'(?P<guild_id>[0-9]{15,20}|@me)'
+                r'/(?P<channel_id>[0-9]{15,20})/(?P<message_id>[0-9]{15,20})/?$'
+            )
+
+            match = url_regex.match(message_url)
+
+            if match is None:
+                if channel_id is None and message_id is None:
+                    return None
+            else:
+                data = match.groupdict()
+                channel_id = data['channel_id']
+                message_id = data['message_id']
+
+        channel = self.get_partial_messageable(channel_id, type=channel_type)
+        return PartialMessage(channel=channel, id=message_id)
+
     def get_stage_instance(self, id: int, /) -> Optional[StageInstance]:
         """Returns a stage instance with the given stage channel ID.
 
@@ -879,7 +970,7 @@ class Client:
             self,
             event: Literal["message"],
             *,
-            check: Optional[Callable[..., bool]] = None,
+            check: Optional[Callable[[Message], bool]] = None,
             timeout: Optional[float] = None,
             ) -> Message:
         ...
@@ -889,7 +980,7 @@ class Client:
             self,
             event: Literal["reaction_add"],
             *,
-            check: Optional[Callable[..., bool]] = None,
+            check: Optional[Callable[[Reaction, Union[User, Member]], bool]] = None,
             timeout: Optional[float] = None,
             ) -> Tuple[Reaction, Union[User, Member]]:
         ...
@@ -899,7 +990,7 @@ class Client:
             self,
             event: Literal["raw_reaction_add"],
             *,
-            check: Optional[Callable[..., bool]] = None,
+            check: Optional[Callable[[RawReactionActionEvent], bool]] = None,
             timeout: Optional[float] = None,
             ) -> RawReactionActionEvent:
         ...
@@ -909,7 +1000,7 @@ class Client:
             self,
             event: Literal["reaction_remove"],
             *,
-            check: Optional[Callable[..., bool]] = None,
+            check: Optional[Callable[[Reaction, Union[User, Member]], bool]] = None,
             timeout: Optional[float] = None,
             ) -> Tuple[Reaction, Union[User, Member]]:
         ...
@@ -919,7 +1010,7 @@ class Client:
             self,
             event: Literal["raw_reaction_remove"],
             *,
-            check: Optional[Callable[..., bool]] = None,
+            check: Optional[Callable[[RawReactionActionEvent], bool]] = None,
             timeout: Optional[float] = None,
             ) -> RawReactionActionEvent:
         ...
@@ -929,9 +1020,9 @@ class Client:
             self,
             event: Literal["slash_command"],
             *,
-            check: Optional[Callable[..., bool]] = None,
+            check: Optional[Callable[[CommandInteraction], bool]] = None,
             timeout: Optional[float] = None,
-            ) -> Interaction[None]:
+            ) -> CommandInteraction:
         ...
 
     @overload
@@ -939,9 +1030,9 @@ class Client:
             self,
             event: Literal["component_interaction"],
             *,
-            check: Optional[Callable[..., bool]] = None,
+            check: Optional[Callable[[ComponentInteraction], bool]] = None,
             timeout: Optional[float] = None,
-            ) -> Interaction[str]:
+            ) -> ComponentInteraction:
         ...
 
     @overload
@@ -949,9 +1040,9 @@ class Client:
             self,
             event: Literal["autocomplete_interaction"],
             *,
-            check: Optional[Callable[..., bool]] = None,
+            check: Optional[Callable[[AutocompleteInteraction], bool]] = None,
             timeout: Optional[float] = None,
-            ) -> Interaction[None]:
+            ) -> AutocompleteInteraction:
         ...
 
     @overload
@@ -959,9 +1050,9 @@ class Client:
             self,
             event: Literal["modal_submit"],
             *,
-            check: Optional[Callable[..., bool]] = None,
+            check: Optional[Callable[[ModalInteraction], bool]] = None,
             timeout: Optional[float] = None,
-            ) -> Interaction[str]:
+            ) -> ModalInteraction:
         ...
 
     @overload
@@ -969,11 +1060,21 @@ class Client:
             self,
             event: Literal["interaction"],
             *,
-            check: Optional[Callable[..., bool]] = None,
+            check: Optional[Callable[[Interaction], bool]] = None,
             timeout: Optional[float] = None,
             ) -> Interaction:
         ...
 
+    @overload
+    async def wait_for(
+            self,
+            event: str,
+            *,
+            check: Optional[Callable[..., bool]] = None,
+            timeout: Optional[float] = None,
+            ) -> Any:
+        ...
+
     def wait_for(
         self,
         event: str,
@@ -1694,11 +1795,16 @@ class Client:
             data = await self.http.bulk_create_global_application_commands(application_id, [c.to_json() for c in commands])
         return [ApplicationCommand.from_data(c) for c in data]
 
-    async def fetch_global_application_commands(self) -> List[ApplicationCommand]:
+    async def fetch_global_application_commands(self, with_localizations: bool = False) -> List[ApplicationCommand]:
         """
         |coro|
 
         Get all of the application commands that the bot has registered globally.
+        
+        Parameters
+        -----------
+        with_localizations: Optional[:class:`bool`]
+            Whether to return all the command localizations.
 
         Returns
         --------
@@ -1715,7 +1821,7 @@ class Client:
         if application_id is None:
             application_info = await self.application_info()
             application_id = application_info.id
-        payload = await self.http.get_global_application_commands(application_id)
+        payload = await self.http.get_global_application_commands(application_id, with_localizations)
         return [
             ApplicationCommand.from_data(i)
             for i in payload
